@@ -40,9 +40,8 @@ static int compute_extensions_start(asn1p_expr_t *expr);
 static int expr_break_recursion(arg_t *arg, asn1p_expr_t *expr);
 static int expr_as_xmlvaluelist(arg_t *arg, asn1p_expr_t *expr);
 static int expr_elements_count(arg_t *arg, asn1p_expr_t *expr);
-static int emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *type);
-static int emit_single_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr);
-static int emit_members_PER_constraints(arg_t *arg);
+static int emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, int juscountvalues, char *type);
+static int emit_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr, const char *pfx);
 static int emit_member_table(arg_t *arg, asn1p_expr_t *expr);
 static int emit_tag2member_map(arg_t *arg, tag2el_t *tag2el, int tag2el_count, const char *opt_modifier);
 static int emit_include_dependencies(arg_t *arg);
@@ -223,9 +222,28 @@ asn1c_lang_C_type_common_INTEGER(arg_t *arg) {
 			OUT("0,\t/* Enumeration is not extensible */\n");
 		}
 		if(expr->expr_type == ASN_BASIC_ENUMERATED)
-			OUT("1\t/* Strict enumeration */\n");
+			OUT("1,\t/* Strict enumeration */\n");
 		else
-			OUT("0\n");
+			OUT("0,\n");
+		OUT("0,\t/* Native long size */\n");
+		OUT("0\n");
+		INDENT(-1);
+		OUT("};\n");
+	}
+
+	if(expr->expr_type == ASN_BASIC_INTEGER
+	&& asn1c_type_fits_long(arg, expr) == FL_FITS_UNSIGN) {
+		REDIR(OT_STAT_DEFS);
+		OUT("static asn_INTEGER_specifics_t asn_SPC_%s_specs_%d = {\n",
+			MKID(expr), expr->_type_unique_index);
+		INDENT(+1);
+		OUT("0,\t");
+		OUT("0,\t");
+		OUT("0,\t");
+		OUT("0,\t");
+		OUT("0,\n");
+		OUT("0,\t/* Native long size */\n");
+		OUT("1\t/* Unsigned representation */\n");
 		INDENT(-1);
 		OUT("};\n");
 	}
@@ -310,8 +328,8 @@ asn1c_lang_C_type_SEQUENCE_def(arg_t *arg) {
 	asn1p_expr_t *expr = arg->expr;
 	asn1p_expr_t *v;
 	int elements;	/* Number of elements */
-	int ext_start = -1;
-	int ext_stop = -1;
+	int ext_start = -2;
+	int ext_stop = -2;
 	tag2el_t *tag2el = NULL;
 	int tag2el_count = 0;
 	int tags_count;
@@ -340,8 +358,6 @@ asn1c_lang_C_type_SEQUENCE_def(arg_t *arg) {
 	if(expr_elements_count(arg, expr)) {
 		int comp_mode = 0;	/* {root,ext=1,root,root,...} */
 
-		if(emit_members_PER_constraints(arg))
-			return -1;
 		OUT("static asn_TYPE_member_t asn_MBR_%s_%d[] = {\n",
 			MKID(expr), expr->_type_unique_index);
 
@@ -449,9 +465,9 @@ asn1c_lang_C_type_SEQUENCE_def(arg_t *arg) {
 		OUT("0, 0, 0,\t/* Optional elements (not needed) */\n");
 	}
 	OUT("%d,\t/* Start extensions */\n",
-			ext_start);
+			ext_start<0 ? -1 : ext_start);
 	OUT("%d\t/* Stop extensions */\n",
-			(ext_stop<ext_start)?elements+1:ext_stop, ext_stop);
+			(ext_stop<ext_start)?elements+1:(ext_stop<0?-1:ext_stop));
 	INDENT(-1);
 	OUT("};\n");
 
@@ -580,8 +596,6 @@ asn1c_lang_C_type_SET_def(arg_t *arg) {
 	if(expr_elements_count(arg, expr)) {
 		int comp_mode = 0;	/* {root,ext=1,root,root,...} */
 
-		if(emit_members_PER_constraints(arg))
-			return -1;
 		OUT("static asn_TYPE_member_t asn_MBR_%s_%d[] = {\n",
 			MKID(expr), expr->_type_unique_index);
 	
@@ -782,8 +796,6 @@ asn1c_lang_C_type_SEx_OF_def(arg_t *arg, int seq_of) {
 	/*
 	 * Print out the table according to which parsing is performed.
 	 */
-	if(emit_members_PER_constraints(arg))
-		return -1;
 	OUT("static asn_TYPE_member_t asn_MBR_%s_%d[] = {\n",
 		MKID(expr), expr->_type_unique_index);
 	INDENT(+1);
@@ -932,8 +944,6 @@ asn1c_lang_C_type_CHOICE_def(arg_t *arg) {
 	 */
 	if(expr_elements_count(arg, expr)) {
 
-		if(emit_members_PER_constraints(arg))
-			return -1;
 		OUT("static asn_TYPE_member_t asn_MBR_%s_%d[] = {\n",
 			MKID(expr), expr->_type_unique_index);
 
@@ -1121,7 +1131,10 @@ asn1c_lang_C_type_SIMPLE_TYPE(arg_t *arg) {
 	if((expr->expr_type == ASN_BASIC_ENUMERATED)
 	|| (0 /* -- prohibited by X.693:8.3.4 */
 		&& expr->expr_type == ASN_BASIC_INTEGER
-		&& expr_elements_count(arg, expr)))
+		&& expr_elements_count(arg, expr))
+	|| (expr->expr_type == ASN_BASIC_INTEGER
+		&& asn1c_type_fits_long(arg, expr) == FL_FITS_UNSIGN)
+	)
 		etd_spec = ETD_HAS_SPECIFICS;
 	else
 		etd_spec = ETD_NO_SPECIFICS;
@@ -1157,16 +1170,6 @@ asn1c_lang_C_type_SIMPLE_TYPE(arg_t *arg) {
 		return 0;
 	}
 
-	REDIR(OT_STAT_DEFS);
-
-	/*
-	 * Print out asn_DEF_<type>_[all_]tags[] vectors.
-	 */
-	tv_mode = emit_tags_vectors(arg, expr, &tags_count, &all_tags_count);
-
-	emit_type_DEF(arg, expr, tv_mode, tags_count, all_tags_count,
-		0, etd_spec);
-
 	REDIR(OT_CODE);
 
 	/*
@@ -1182,6 +1185,7 @@ asn1c_lang_C_type_SIMPLE_TYPE(arg_t *arg) {
 		INDENT(+1);
 		OUT("\t\tasn_app_constraint_failed_f *ctfailcb, void *app_key) {");
 		OUT("\n");
+		DEBUG("expr constraint checking code for %s", p);
 		if(asn1c_emit_constraint_checking_code(arg) == 1) {
 			OUT("/* Replace with underlying type checker */\n");
 			OUT("td->check_constraints "
@@ -1195,6 +1199,20 @@ asn1c_lang_C_type_SIMPLE_TYPE(arg_t *arg) {
 		OUT("\n");
 	}
 
+	REDIR(OT_STAT_DEFS);
+
+	/*
+	 * Print out asn_DEF_<type>_[all_]tags[] vectors.
+	 */
+	tv_mode = emit_tags_vectors(arg, expr, &tags_count, &all_tags_count);
+	DEBUG("emit tag vectors for %s %d, %d, %d", expr->Identifier,
+		tv_mode, tags_count, all_tags_count);
+
+	emit_type_DEF(arg, expr, tv_mode, tags_count, all_tags_count,
+		0, etd_spec);
+
+	REDIR(OT_CODE);
+
 	/*
 	 * Emit suicidal functions.
 	 */
@@ -1588,7 +1606,7 @@ _add_tag2el_member(arg_t *arg, tag2el_t **tag2el, int *count, int el_no, fte_e f
 		if(p)	*tag2el = p;
 		else	return -1;
 
-		DEBUG("Found tag for %s: %ld",
+		if(0) DEBUG("Found tag for %s: %ld",
 			arg->expr->Identifier,
 			(long)tag.tag_value);
 
@@ -1678,14 +1696,17 @@ emit_tags_vectors(arg_t *arg, asn1p_expr_t *expr, int *tags_count_r, int *all_ta
 
 	/* Fetch a chain of tags */
 	tags_count = asn1f_fetch_tags(arg->asn, expr->module, expr, &tags, 0);
-	if(tags_count < 0)
+	if(tags_count < 0) {
+		DEBUG("fail to fetch tags for %s", expr->Identifier);
 		return -1;
+	}
 
 	/* Fetch a chain of tags */
 	all_tags_count = asn1f_fetch_tags(arg->asn, expr->module, expr,
 		&all_tags, AFT_FULL_COLLECT);
 	if(all_tags_count < 0) {
 		if(tags) free(tags);
+		DEBUG("fail to fetch tags chain for %s", expr->Identifier);
 		return -1;
 	}
 
@@ -1772,8 +1793,24 @@ expr_get_type(arg_t *arg, asn1p_expr_t *expr) {
 	return A1TC_INVALID;
 }
 
+static asn1c_integer_t
+PER_FROM_alphabet_characters(asn1cnst_range_t *range) {
+	asn1c_integer_t numchars = 0;
+	if(range->el_count) {
+		int i;
+		for(i = 0; i < range->el_count; i++)
+			numchars
+			+= PER_FROM_alphabet_characters(range->elements[i]);
+	} else {
+		assert(range->left.type == ARE_VALUE);
+		assert(range->right.type == ARE_VALUE);
+		numchars = 1 + (range->right.value - range->left.value);
+	}
+	return numchars;
+}
+
 static int
-emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *type) {
+emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, int alphabetsize, char *type) {
 	if(!range || range->incompatible || range->not_PER_visible) {
 		OUT("{ APC_UNCONSTRAINED,\t-1, -1,  0,  0 }");
 		return 0;
@@ -1790,6 +1827,11 @@ emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *typ
 			if(range->empty_constraint)
 				r = 0;
 
+			if(alphabetsize) {
+				/* X.691: 27.5.2 */
+				r = PER_FROM_alphabet_characters(range);
+			}
+
 			/* Compute real constraint */
 			for(rbits = 0; rbits < (8 * sizeof(r)); rbits++) {
 				if(r <= cover)
@@ -1805,14 +1847,16 @@ emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *typ
 				}
 			}
 
-			if(1) {
+			if(alphabetsize) {
+				ebits = rbits;
+			} else {
 				/* X.691, #10.9.4.1 */
 				for(ebits = 0; ebits <= 16; ebits++)
 					if(r <= 1 << ebits) break;
 				if(ebits == 17
 				|| range->right.value >= 65536)
 					ebits = -1;
-			} else {
+			if(0) {
 				/* X.691, #10.5.7.1 */
 				for(ebits = 0; ebits <= 8; ebits++)
 					if(r <= 1 << ebits) break;
@@ -1823,10 +1867,24 @@ emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *typ
 						ebits = -1;
 				}
 			}
+			}
+
 			OUT("{ APC_CONSTRAINED%s,%s% d, % d, ",
 				range->extensible
 					? " | APC_EXTENSIBLE" : "",
 				range->extensible ? " " : "\t", rbits, ebits);
+
+			if(alphabetsize) {
+				asn1c_integer_t lv = range->left.value;
+				asn1c_integer_t rv = range->right.value;
+				int gcmt = 0;
+				if(lv > 0x7fffffff) { lv = 0x7fffffff; gcmt++; }
+				if(rv > 0x7fffffff) { rv = 0x7fffffff; gcmt++; }
+				if(gcmt) {
+					OINTS(lv); OUT(", "); OINTS(rv); OUT(" }");
+					goto pcmt;
+				}
+			}
 		} else {
 			if(range->extensible) {
 				OUT("{ APC_SEMI_CONSTRAINED | APC_EXTENSIBLE, "
@@ -1835,12 +1893,14 @@ emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *typ
 				OUT("{ APC_SEMI_CONSTRAINED,\t-1, -1, ");
 			}
 		}
-		OUT("% " PRIdASN ", % " PRIdASN " }",
-			range->left.value, range->right.value);
+		OINTS(range->left.value); OUT(", ");
+		OINTS(range->right.value); OUT(" }");
 	} else {
 		OUT("{ APC_UNCONSTRAINED,\t-1, -1,  0,  0 }");
 	}
 
+  pcmt:
+
 	/*
 	 * Print some courtesy debug information.
 	 */
@@ -1867,12 +1927,29 @@ emit_single_member_PER_constraint(arg_t *arg, asn1cnst_range_t *range, char *typ
 }
 
 static int
-emit_single_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr) {
+emit_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr, const char *pfx) {
+	int save_target = arg->target->target;
 	asn1cnst_range_t *range;
 	asn1p_expr_type_e etype;
 
 	etype = expr_get_type(arg, expr);
 
+	if((arg->flags & A1C_GEN_PER)
+	&& (expr->constraints
+		|| etype == ASN_BASIC_ENUMERATED
+		|| etype == ASN_CONSTR_CHOICE)
+	) {
+		/* Fall through */
+	} else {
+		return 0;
+	}
+
+	REDIR(OT_CTDEFS);
+
+	OUT("static asn_per_constraints_t "
+		"asn_PER_%s_%s_constr_%d = {\n",
+		pfx, MKID(expr), expr->_type_unique_index);
+
 	INDENT(+1);
 
 	/*
@@ -1903,13 +1980,43 @@ emit_single_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr) {
 		tmprng.left.value = 0;
 		tmprng.right.type = ARE_VALUE;
 		tmprng.right.value = eidx < 0 ? 0 : eidx;
-		if(emit_single_member_PER_constraint(arg, &tmprng, 0))
+		if(emit_single_member_PER_constraint(arg, &tmprng, 0, 0))
 			return -1;
+	} else if(etype & ASN_STRING_KM_MASK) {
+		range = asn1constraint_compute_PER_range(etype,
+				expr->combined_constraints, ACT_CT_FROM,
+				0, 0, 0);
+		DEBUG("Emitting FROM constraint for %s", expr->Identifier);
+
+		if((range->left.type == ARE_MIN && range->right.type == ARE_MAX)
+		|| range->not_PER_visible) {
+			switch(etype) {
+			case ASN_STRING_BMPString:
+				range->left.type = ARE_VALUE;
+				range->left.value = 0;
+				range->right.type = ARE_VALUE;
+				range->right.value = 65535;
+				range->not_PER_visible = 0;
+				range->extensible = 0;
+				break;
+			case ASN_STRING_UniversalString:
+				OUT("{ APC_CONSTRAINED,\t32, 32,"
+					" 0, 2147483647 }"
+					" /* special case 1 */\n");
+				goto avoid;
+			default:
+				break;
+			}
+		}
+		if(emit_single_member_PER_constraint(arg, range, 1, 0))
+			return -1;
+		avoid:
+		asn1constraint_range_free(range);
 	} else {
 		range = asn1constraint_compute_PER_range(etype,
 				expr->combined_constraints, ACT_EL_RANGE,
 				0, 0, 0);
-		if(emit_single_member_PER_constraint(arg, range, 0))
+		if(emit_single_member_PER_constraint(arg, range, 0, 0))
 			return -1;
 		asn1constraint_range_free(range);
 	}
@@ -1917,38 +2024,86 @@ emit_single_member_PER_constraints(arg_t *arg, asn1p_expr_t *expr) {
 
 	range = asn1constraint_compute_PER_range(etype,
 			expr->combined_constraints, ACT_CT_SIZE, 0, 0, 0);
-	if(emit_single_member_PER_constraint(arg, range, "SIZE"))
+	if(emit_single_member_PER_constraint(arg, range, 0, "SIZE"))
 		return -1;
 	asn1constraint_range_free(range);
-	OUT("\n");
+	OUT(",\n");
+
+	if((etype & ASN_STRING_KM_MASK) && (expr->_mark & TM_PERFROMCT)) {
+		int old_target = arg->target->target;
+		REDIR(OT_CODE);
+
+		OUT("static int asn_PER_MAP_%s_%d_v2c(unsigned int value) {\n",
+			MKID(expr), expr->_type_unique_index);
+		OUT("\tif(value >= sizeof(permitted_alphabet_table_%d)/"
+			"sizeof(permitted_alphabet_table_%d[0]))\n",
+			expr->_type_unique_index,
+			expr->_type_unique_index);
+		OUT("\t\treturn -1;\n");
+		OUT("\treturn permitted_alphabet_table_%d[value] - 1;\n",
+			expr->_type_unique_index);
+		OUT("}\n");
+
+		OUT("static int asn_PER_MAP_%s_%d_c2v(unsigned int code) {\n",
+			MKID(expr), expr->_type_unique_index);
+		OUT("\tif(code >= sizeof(permitted_alphabet_code2value_%d)/"
+			"sizeof(permitted_alphabet_code2value_%d[0]))\n",
+			expr->_type_unique_index,
+			expr->_type_unique_index);
+		OUT("\t\treturn -1;\n");
+		OUT("\treturn permitted_alphabet_code2value_%d[code];\n",
+			expr->_type_unique_index);
+		OUT("}\n");
+
+		REDIR(old_target);
+
+		OUT("asn_PER_MAP_%s_%d_v2c,\t/* Value to PER code map */\n",
+			MKID(expr), expr->_type_unique_index);
+		OUT("asn_PER_MAP_%s_%d_c2v\t/* PER code to value map */\n",
+			MKID(expr), expr->_type_unique_index);
+	} else if(etype & ASN_STRING_KM_MASK) {
+		DEBUG("No PER value map necessary for %s", MKID(expr));
+		OUT("0, 0\t/* No PER character map necessary */\n");
+	} else {
+		OUT("0, 0\t/* No PER value map */\n");
+	}
 
 	INDENT(-1);
 
+	OUT("};\n");
+
+	REDIR(save_target);
+
 	return 0;
 }
 
 static int
-emit_members_PER_constraints(arg_t *arg) {
-	asn1p_expr_t *expr = arg->expr;
-	asn1p_expr_t *v;
-
-	if(!(arg->flags & A1C_GEN_PER))
-		return 0;
-
-	TQ_FOR(v, &(expr->members), next) {
-		if(v->constraints
-		|| v->expr_type == ASN_BASIC_ENUMERATED
-		|| v->expr_type == ASN_CONSTR_CHOICE) {
-			OUT("static asn_per_constraints_t "
-				"asn_PER_memb_%s_constr_%d = {\n",
-				MKID(v), v->_type_unique_index);
-			if(emit_single_member_PER_constraints(arg, v))
-				return -1;
-			OUT("};\n");
-		}
+safe_string(const uint8_t *buf, int size) {
+	const uint8_t *end = buf + size;
+	for(; buf < end; buf++) {
+		int ch = *buf;
+		if((ch < 0x20 || ch > 0x7e) || ch == '"')
+			return 0;
 	}
+	return 1;
+}
 
-	return 0;
+static void
+emit_default_value(arg_t *arg, asn1p_value_t *v) {
+
+	OUT("static uint8_t defv[] = ");
+	assert(v->type == ATV_STRING);
+
+	if(safe_string(v->value.string.buf, v->value.string.size)) {
+		OUT("\"%s\";\n", v->value.string.buf);
+	} else {
+		uint8_t *b = v->value.string.buf;
+		uint8_t *e = v->value.string.size + b;
+		OUT("{ ");
+		for(;b < e; b++)
+			OUT("0x%02x, ", *b);
+		OUT("0 };\n");
+	}
 }
 
 static int
@@ -1969,7 +2124,14 @@ try_inline_default(arg_t *arg, asn1p_expr_t *expr, int out) {
 			fits_long = asn1c_type_fits_long(arg, expr)!=FL_NOTFIT;
 		if(fits_long && !expr->marker.default_value->value.v_integer)
 			expr->marker.flags &= ~EM_INDIRECT;
-		if(!out) return 1;
+		if(!out) {
+			OUT("asn_DFL_%d_set_%" PRIdASN
+				",\t/* DEFAULT %" PRIdASN " */\n",
+				expr->_type_unique_index,
+				expr->marker.default_value->value.v_integer,
+				expr->marker.default_value->value.v_integer);
+			return 1;
+		}
 		REDIR(OT_STAT_DEFS);
 		OUT("static int asn_DFL_%d_set_%" PRIdASN "(int set_value, void **sptr) {\n",
 			expr->_type_unique_index,
@@ -1988,12 +2150,14 @@ try_inline_default(arg_t *arg, asn1p_expr_t *expr, int out) {
 		OUT("/* Install default value %" PRIdASN " */\n",
 			expr->marker.default_value->value.v_integer);
 		if(fits_long) {
-			OUT("*st = %" PRIdASN ";\n",
-				expr->marker.default_value->value.v_integer);
+			OUT("*st = ");
+			OINT(expr->marker.default_value->value.v_integer);
+			OUT(";\n");
 			OUT("return 0;\n");
 		} else {
-			OUT("return asn_long2INTEGER(st, %" PRIdASN ");\n",
-				expr->marker.default_value->value.v_integer);
+			OUT("return asn_long2INTEGER(st, ");
+			OINT(expr->marker.default_value->value.v_integer);
+			OUT(");\n");
 		}
 		INDENT(-1);
 		OUT("} else {\n");
@@ -2020,7 +2184,52 @@ try_inline_default(arg_t *arg, asn1p_expr_t *expr, int out) {
 		//expr->marker.flags &= ~EM_INDIRECT;
 		return 0;
 	default:
-		break;
+	  if(etype & ASN_STRING_KM_MASK) {
+		if(expr->marker.default_value == NULL
+		|| expr->marker.default_value->type != ATV_STRING)
+			break;
+		if(!out) {
+			OUT("asn_DFL_%d_set,\t/* DEFAULT \"%s\" */\n",
+				expr->_type_unique_index,
+				expr->marker.default_value->value.string.buf);
+			return 1;
+		}
+		REDIR(OT_STAT_DEFS);
+		OUT("static int asn_DFL_%d_set(int set_value, void **sptr) {\n", expr->_type_unique_index);
+		INDENT(+1);
+		emit_default_value(arg, expr->marker.default_value);
+		OUT("%s *st = *sptr;\n", asn1c_type_name(arg, expr, TNF_CTYPE));
+		OUT("\n");
+		OUT("if(!st) {\n");
+		OUT("\tif(!set_value) return -1;\t/* Not a default value */\n");
+		OUT("\tst = (*sptr = CALLOC(1, sizeof(*st)));\n");
+		OUT("\tif(!st) return -1;\n");
+		OUT("}\n");
+		OUT("\n");
+		OUT("if(set_value) {\n");
+		INDENT(+1);
+			OUT("uint8_t *ptr = MALLOC(sizeof(defv));\n");
+			OUT("if(!ptr) return -1;\n");
+			OUT("memcpy(ptr, &defv, sizeof(defv));\n");
+			OUT("FREEMEM(st->buf);\n");
+			OUT("st->buf = ptr;\n");
+			OUT("st->size = sizeof(defv) - 1;\n");
+			OUT("return 0;\n");
+		INDENT(-1);
+		OUT("} else {\n");
+		INDENT(+1);
+			OUT("if(st->size != (sizeof(defv) - 1)\n");
+			OUT("|| memcmp(st->buf, &defv, sizeof(defv) - 1))\n");
+			OUT("\treturn 0;\n");
+			OUT("return 1;\n");
+		INDENT(-1);
+		OUT("}\n"); OUT("\n");
+		INDENT(-1);
+		OUT("}\n");
+		REDIR(save_target);
+		return 1;
+	  }
+	  break;
 	}
 	return 0;
 }
@@ -2086,7 +2295,9 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 
 	OUT(",\n");
 	if(C99_MODE) OUT(".tag_mode = ");
-	if(expr->tag.tag_class) {
+	if((!(expr->expr_type &  ASN_CONSTR_MASK)
+	   || expr->expr_type == ASN_CONSTR_CHOICE)
+	&& expr->tag.tag_class) {
 		if(expr->tag.tag_mode == TM_IMPLICIT)
 		OUT("-1,\t/* IMPLICIT tag at current level */\n");
 		else
@@ -2100,7 +2311,9 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 		|| expr->expr_type == ASN_BASIC_ENUMERATED
 		|| (0 /* -- prohibited by X.693:8.3.4 */
 			&& expr->expr_type == ASN_BASIC_INTEGER
-			&& expr_elements_count(arg, expr));
+			&& expr_elements_count(arg, expr))
+		|| (expr->expr_type == ASN_BASIC_INTEGER
+			&& asn1c_type_fits_long(arg, expr) == FL_FITS_UNSIGN);
 	if(C99_MODE) OUT(".type = ");
 	OUT("&asn_DEF_");
 	if(complex_contents) {
@@ -2128,9 +2341,7 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 	}
 	if(C99_MODE) OUT(".per_constraints = ");
 	if(arg->flags & A1C_GEN_PER) {
-		if(expr->constraints
-		|| expr->expr_type == ASN_BASIC_ENUMERATED
-		|| expr->expr_type == ASN_CONSTR_CHOICE) {
+		if(expr->constraints) {
 			OUT("&asn_PER_memb_%s_constr_%d,\n",
 				MKID(expr),
 				expr->_type_unique_index);
@@ -2142,11 +2353,6 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 	}
 	if(C99_MODE) OUT(".default_value = ");
 	if(try_inline_default(arg, expr, 0)) {
-		OUT("asn_DFL_%d_set_%" PRIdASN
-				",\t/* DEFAULT %" PRIdASN " */\n",
-			expr->_type_unique_index,
-			expr->marker.default_value->value.v_integer,
-			expr->marker.default_value->value.v_integer);
 	} else {
 		OUT("0,\n");
 	}
@@ -2175,6 +2381,7 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 	OUT("\t\tasn_app_constraint_failed_f *ctfailcb, void *app_key) {\n");
 	tmp_arg = *arg;
 	tmp_arg.expr = expr;
+	DEBUG("member constraint checking code for %s", p);
 	if(asn1c_emit_constraint_checking_code(&tmp_arg) == 1) {
 		OUT("return td->check_constraints"
 			"(td, sptr, ctfailcb, app_key);\n");
@@ -2183,6 +2390,9 @@ emit_member_table(arg_t *arg, asn1p_expr_t *expr) {
 	OUT("}\n");
 	OUT("\n");
 
+	if(emit_member_PER_constraints(arg, expr, "memb"))
+		return -1;
+
 	REDIR(save_target);
 
 	return 0;
@@ -2199,17 +2409,8 @@ emit_type_DEF(arg_t *arg, asn1p_expr_t *expr, enum tvm_compat tv_mode, int tags_
 
 	terminal = asn1f_find_terminal_type_ex(arg->asn, expr);
 
-	if((arg->flags & A1C_GEN_PER)
-	&& (expr->constraints
-		|| expr->expr_type == ASN_BASIC_ENUMERATED
-		|| expr->expr_type == ASN_CONSTR_CHOICE)
-	) {
-		OUT("static asn_per_constraints_t asn_PER_%s_constr_%d = {\n",
-			p, expr->_type_unique_index);
-		if(emit_single_member_PER_constraints(arg, expr))
-			return -1;
-		OUT("};\n");
-	}
+	if(emit_member_PER_constraints(arg, expr, "type"))
+		return -1;
 
 	if(HIDE_INNER_DEFS)
 		OUT("static /* Use -fall-defs-global to expose */\n");
@@ -2305,7 +2506,7 @@ emit_type_DEF(arg_t *arg, asn1p_expr_t *expr, enum tvm_compat tv_mode, int tags_
 			if(expr->constraints
 			|| expr->expr_type == ASN_BASIC_ENUMERATED
 			|| expr->expr_type == ASN_CONSTR_CHOICE) {
-				OUT("&asn_PER_%s_constr_%d,\n",
+				OUT("&asn_PER_type_%s_constr_%d,\n",
 					p, expr->_type_unique_index);
 			} else {
 				OUT("0,\t/* No PER visible constraints */\n");
