@@ -12,6 +12,17 @@ static int emit_value_determination_code(arg_t *arg, asn1p_expr_type_e etype, as
 static int emit_size_determination_code(arg_t *arg, asn1p_expr_type_e etype);
 static asn1p_expr_type_e _find_terminal_type(arg_t *arg);
 static int emit_range_comparison_code(arg_t *arg, asn1cnst_range_t *range, const char *varname, asn1c_integer_t natural_start, asn1c_integer_t natural_stop);
+static int native_long_sign(asn1cnst_range_t *r);	/* -1, 0, 1 */
+
+static int
+ulong_optimization(asn1p_expr_type_e etype, asn1cnst_range_t *r_size,
+						asn1cnst_range_t *r_value)
+{
+	return (!r_size && r_value
+		&& (etype == ASN_BASIC_INTEGER
+		|| etype == ASN_BASIC_ENUMERATED)
+		&& native_long_sign(r_value) == 0);
+}
 
 int
 asn1c_emit_constraint_checking_code(arg_t *arg) {
@@ -23,6 +34,7 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 	int got_something = 0;
 	int alphabet_table_compiled;
 	int produce_st = 0;
+	int ulong_optimize = 0;
 
 	ct = expr->combined_constraints;
 	if(ct == NULL)
@@ -31,7 +43,7 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 	etype = _find_terminal_type(arg);
 
 	r_value=asn1constraint_compute_PER_range(etype, ct, ACT_EL_RANGE,0,0,0);
-	r_size = asn1constraint_compute_PER_range(etype, ct, ACT_CT_SIZE,0,0,0);
+	r_size =asn1constraint_compute_PER_range(etype, ct, ACT_CT_SIZE, 0,0,0);
 	if(r_value) {
 		if(r_value->incompatible
 		|| r_value->empty_constraint
@@ -72,7 +84,7 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 	case ASN_BASIC_BIT_STRING:
 	case ASN_BASIC_OCTET_STRING:
 		produce_st = 1;
-			break;
+		break;
 	default:
 		if(etype & ASN_STRING_MASK)
 			produce_st = 1;
@@ -91,7 +103,14 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 			switch(etype) {
 			case ASN_BASIC_INTEGER:
 			case ASN_BASIC_ENUMERATED:
-				OUT("long value;\n");
+				if(native_long_sign(r_value) >= 0) {
+					ulong_optimize = ulong_optimization(etype, r_size, r_value);
+					if(!ulong_optimize) {
+						OUT("unsigned long value;\n");
+					}
+				} else {
+					OUT("long value;\n");
+				}
 				break;
 			case ASN_BASIC_REAL:
 				OUT("double value;\n");
@@ -119,7 +138,7 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 	OUT("}\n");
 	OUT("\n");
 
-	if(r_value)
+	if((r_value) && (!ulong_optimize))
 		emit_value_determination_code(arg, etype, r_value);
 	if(r_size)
 		emit_size_determination_code(arg, etype);
@@ -133,7 +152,17 @@ asn1c_emit_constraint_checking_code(arg_t *arg) {
 	INDENT(+1);
 
 	/*
-	 * Here is an if() {} else {} constaint checking code.
+	 * Optimization for unsigned longs.
+	 */
+	if(ulong_optimize) {
+		OUT("\n");
+		OUT("/* Constraint check succeeded */\n");
+		OUT("return 0;\n");
+		return 0;
+	}
+
+	/*
+	 * Here is an if() {} else {} consrtaint checking code.
 	 */
 	OUT("\n");
 	OUT("if(");
@@ -222,7 +251,6 @@ asn1c_emit_constraint_tables(arg_t *arg, int got_size) {
 		return 0;
 	}
 
-
 	if(range->left.type == ARE_MIN
 	&& range->right.type == ARE_MAX) {
 		/*
@@ -251,7 +279,7 @@ asn1c_emit_constraint_tables(arg_t *arg, int got_size) {
 	if(range->el_count == 0) {
 		/*
 		 * It's better to have a short if() check
-		 * than waste 4k of table space
+		 * than waste 1k of table space
 		 */
 		use_table = 0;
 	}
@@ -265,6 +293,7 @@ asn1c_emit_constraint_tables(arg_t *arg, int got_size) {
 	}
 
 	if(use_table) {
+		int cardinal = 0;
 		int i, n = 0;
 		int untl;
 		memset(table, 0, sizeof(table));
@@ -289,10 +318,11 @@ asn1c_emit_constraint_tables(arg_t *arg, int got_size) {
 		OUT("static int permitted_alphabet_table_%d[%d] = {\n",
 			arg->expr->_type_unique_index, max_table_size);
 		for(n = 0; n < untl; n++) {
-			OUT("%d,", table[n]?1:0);
+			cardinal += table[n] ? 1 : 0;
+			OUT("%2d,", table[n]);
 			if(!((n+1) % 16)) {
 				int c;
-				if(!n) {
+				if(!n || (n-15) + range_start >= 0x80) {
 					OUT("\n");
 					continue;
 				}
@@ -313,6 +343,24 @@ asn1c_emit_constraint_tables(arg_t *arg, int got_size) {
 			}
 		}
 		OUT("};\n");
+
+		if((arg->flags & A1C_GEN_PER)
+		&& (etype & ASN_STRING_KM_MASK)) {
+		    int c;
+		    OUT("static int permitted_alphabet_code2value_%d[%d] = {\n",
+			arg->expr->_type_unique_index, cardinal);
+		    for(n = c = 0; c < max_table_size; c++) {
+			if(table[c]) {
+				OUT("%d,", c);
+				if(!((++n) % 16)) OUT("\n");
+			}
+		    }
+		    OUT("};\n");
+		    OUT("\n");
+		    DEBUG("code2value map gen for %s", arg->expr->Identifier);
+		    arg->expr->_mark |= TM_PERFROMCT;
+		}
+
 		OUT("\n");
 	} else if(etype == ASN_STRING_UTF8String) {
 		/*
@@ -476,20 +524,20 @@ emit_range_comparison_code(arg_t *arg, asn1cnst_range_t *range, const char *varn
 		}
 
 		if(ignore_left) {
-			OUT("%s <= %" PRIdASN, varname,
-				r->right.value);
+			OUT("%s <= ", varname);
+			OINT(r->right.value);
 		} else if(ignore_right) {
-			OUT("%s >= %" PRIdASN, varname,
-				r->left.value);
+			OUT("%s >= ", varname);
+			OINT(r->left.value);
 		} else if(r->left.value == r->right.value) {
-			OUT("%s == %" PRIdASN, varname,
-				r->right.value);
+			OUT("%s == ", varname);
+			OINT(r->right.value);
 		} else {
-			OUT("%s >= %" PRIdASN " && %s <= %" PRIdASN,
-				varname,
-				r->left.value,
-				varname,
-				r->right.value);
+			OUT("%s >= ", varname);
+			OINT(r->left.value);
+			OUT(" && ");
+			OUT("%s <= ", varname);
+			OINT(r->right.value);
 		}
 		if(r != range) OUT(")");
 		generated_something = 1;
@@ -559,9 +607,15 @@ emit_value_determination_code(arg_t *arg, asn1p_expr_type_e etype, asn1cnst_rang
 	switch(etype) {
 	case ASN_BASIC_INTEGER:
 	case ASN_BASIC_ENUMERATED:
-		if(asn1c_type_fits_long(arg, arg->expr) != FL_NOTFIT) {
+		if(asn1c_type_fits_long(arg, arg->expr) == FL_FITS_UNSIGN) {
+			OUT("value = *(const unsigned long *)sptr;\n");
+		} else if(asn1c_type_fits_long(arg, arg->expr) != FL_NOTFIT) {
 			OUT("value = *(const long *)sptr;\n");
 		} else {
+			/*
+			 * In some cases we can explore our knowledge of
+			 * underlying INTEGER_t->buf format.
+			 */
 			if(r_value->el_count == 0
 			&& (
 				/* Speed-up common case: (0..MAX) */
@@ -579,14 +633,26 @@ emit_value_determination_code(arg_t *arg, asn1p_expr_type_e etype, asn1cnst_rang
 				break;
 			}
 
-			OUT("if(asn_INTEGER2long(st, &value)) {\n");
+			if(native_long_sign(r_value) >= 0) {
+				/* Special case for treating unsigned longs */
+				OUT("if(asn_INTEGER2ulong(st, &value)) {\n");
 				INDENT(+1);
 				OUT("_ASN_CTFAIL(app_key, td, sptr,\n");
 				OUT("\t\"%%s: value too large (%%s:%%d)\",\n");
 				OUT("\ttd->name, __FILE__, __LINE__);\n");
 				OUT("return -1;\n");
 				INDENT(-1);
-			OUT("}\n");
+				OUT("}\n");
+			} else {
+				OUT("if(asn_INTEGER2long(st, &value)) {\n");
+				INDENT(+1);
+				OUT("_ASN_CTFAIL(app_key, td, sptr,\n");
+				OUT("\t\"%%s: value too large (%%s:%%d)\",\n");
+				OUT("\ttd->name, __FILE__, __LINE__);\n");
+				OUT("return -1;\n");
+				INDENT(-1);
+				OUT("}\n");
+			}
 		}
 		break;
 	case ASN_BASIC_REAL:
@@ -633,3 +699,20 @@ _find_terminal_type(arg_t *arg) {
 	return A1TC_INVALID;
 }
 
+static int
+native_long_sign(asn1cnst_range_t *r) {
+	if(r->left.type == ARE_VALUE
+	&& r->left.value >= 0
+	&& r->right.type == ARE_VALUE
+	&& r->right.value > 2147483647UL
+	&& r->right.value <= 4294967295UL) {
+		if(r->el_count == 0
+		&& r->left.value == 0
+		&& r->right.value == 4294967295UL)
+			return 0;
+		else
+			return 1;
+	} else {
+		return -1;
+	}
+}
