@@ -22,17 +22,18 @@
 #include <unistd.h>		/* for unlink(2) */
 #endif
 
-#ifdef	WIN32
+#ifdef	_WIN32
 #include <io.h>
+#include <malloc.h>
+#include <fcntl.h>
 #define	open	_open
 #define	close	_close
-#define	alloca		_alloca
 #define	snprintf	_snprintf
 #define	vsnprintf	_vsnprintf
 #else
-#include <fcntl.h>		/* for open(2) */
 #include <glob.h>		/* for glob(3) */
 #endif
+#include <fcntl.h>		/* for open(2) */
 
 #ifdef	HAVE_SYS_PARAM_H
 #include <sys/param.h>	/* For MAXPATHLEN */
@@ -51,7 +52,6 @@ typedef struct arg_s {
 	struct compiler_streams *target;
 
 	asn1p_t		*asn;
-	//asn1p_module_t	*mod;
 	asn1p_expr_t	*expr;
 
 	int embed;
