@@ -158,7 +158,8 @@ asn1c_type_name(arg_t *arg, asn1p_expr_t *expr, enum tnfmt _format) {
 		while(top_parent->parent_expr)
 			top_parent = top_parent->parent_expr;
 
-	DEBUG("asn1c_type_name(%s: 0x%x)", expr->Identifier, expr->expr_type);
+	if(0) DEBUG("asn1c_type_name(%s: 0x%x)",
+		expr->Identifier, expr->expr_type);
 
 	switch(expr->expr_type) {
 	case A1TC_REFERENCE:
@@ -214,6 +215,8 @@ asn1c_type_name(arg_t *arg, asn1p_expr_t *expr, enum tnfmt _format) {
 			case TNF_RSAFE:
 				if(expr->expr_type == ASN_BASIC_REAL)
 					return "double";
+				else if(asn1c_type_fits_long(arg, expr) == FL_FITS_UNSIGN)
+					return "unsigned long";
 				else
 					return "long";
 			default:
@@ -258,9 +261,11 @@ asn1c_type_name(arg_t *arg, asn1p_expr_t *expr, enum tnfmt _format) {
 	case TNF_INCLUDE:
 		return asn1c_make_identifier(
 			AMI_MASK_ONLY_SPACES | AMI_NODELIMITER,
-			0, stdname ? "<" : "\"",
+			0, ((!stdname || (arg->flags & A1C_INCLUDES_QUOTED))
+				? "\"" : "<"),
 			exprid ? exprid->Identifier : typename,
-			stdname ? ".h>" : ".h\"", 0);
+			((!stdname || (arg->flags & A1C_INCLUDES_QUOTED))
+				? ".h\"" : ".h>"), 0);
 	case TNF_SAFE:
 		return asn1c_make_identifier(0, exprid, typename, 0);
 	case TNF_CTYPE:	/* C type */
@@ -277,7 +282,7 @@ asn1c_type_name(arg_t *arg, asn1p_expr_t *expr, enum tnfmt _format) {
 
 /*
  * Check whether the specified INTEGER or ENUMERATED type can be represented
- * using the generic 'long' type.
+ * using the generic 'long' or 'unsigned long' type.
  */
 enum asn1c_fitslong_e
 asn1c_type_fits_long(arg_t *arg, asn1p_expr_t *expr) {
@@ -338,7 +343,7 @@ asn1c_type_fits_long(arg_t *arg, asn1p_expr_t *expr) {
 			right = range->right;
 			/* Use 4 instead of sizeof(long) is justified! */
 			if(right.type == ARE_VALUE && right.value <= 4)
-				return FL_FITSOK;
+				return FL_FITS_SIGNED;
 		}
 		asn1constraint_range_free(range);
 	}
@@ -363,6 +368,15 @@ asn1c_type_fits_long(arg_t *arg, asn1p_expr_t *expr) {
 	right = range->right;
 	asn1constraint_range_free(range);
 
+	/* Special case for unsigned */
+	if(left.type == ARE_VALUE
+		&& left.value >= 0
+	&& right.type == ARE_VALUE
+		&& right.value > 2147483647
+		&& right.value <= 4294967295UL)
+		return FL_FITS_UNSIGN;
+		
+
 	/* If some fixed value is outside of target range, not fit */
 	if(left.type == ARE_VALUE
 			&& (left.value < LEFTMIN || left.value > RIGHTMAX))
@@ -377,6 +391,6 @@ asn1c_type_fits_long(arg_t *arg, asn1p_expr_t *expr) {
 			? FL_FORCED : FL_NOTFIT;
 	}
 
-	return FL_FITSOK;
+	return FL_FITS_SIGNED;
 }
 
