@@ -30,12 +30,14 @@ char *asn1c_type_name(arg_t *arg, asn1p_expr_t *expr, enum tnfmt _format);
  * using the generic 'long' type.
  * Return values:
  * 	FL_NOTFIT:	No, it cannot be represented using long.
- * 	FL_FITSOK:	It can be represented using long.
+ * 	FL_FITS_SIGNED:	It can be represented using signed long.
+ * 	FL_FITS_UNSIGN:	It can be represented using unsigned long.
  * 	FL_FORCED:	Probably can't, but -fnative-types is in force.
  */
 enum asn1c_fitslong_e {
 	FL_NOTFIT,
-	FL_FITSOK,
+	FL_FITS_SIGNED,
+	FL_FITS_UNSIGN,
 	FL_FORCED,
 };
 enum asn1c_fitslong_e asn1c_type_fits_long(arg_t *arg, asn1p_expr_t *expr);
