@@ -22,6 +22,7 @@ typedef struct compiler_streams {
 		OT_POST_INCLUDE,/* #include after type definition */
 		OT_CTABLES,	/* Constraint tables */
 		OT_CODE,	/* Some code */
+		OT_CTDEFS,	/* Constraint definitions */
 		OT_STAT_DEFS,	/* Static definitions */
 		OT_MAX
 	} target;
@@ -34,7 +35,7 @@ typedef struct compiler_streams {
 } compiler_streams_t;
 
 static char *_compiler_stream2str[] __attribute__ ((unused))
-    = { "IGNORE", "INCLUDES", "DEPS", "FWD-DECLS", "TYPE-DECLS", "FUNC-DECLS", "POST-INCLUDE", "CTABLES", "CODE", "STAT-DEFS" };
+    = { "IGNORE", "INCLUDES", "DEPS", "FWD-DECLS", "TYPE-DECLS", "FUNC-DECLS", "POST-INCLUDE", "CTABLES", "CODE", "CTDEFS", "STAT-DEFS" };
 
 int asn1c_compiled_output(arg_t *arg, const char *fmt, ...);
 
@@ -82,7 +83,12 @@ int asn1c_compiled_output(arg_t *arg, const char *fmt, ...);
 	} while(0)
 
 /* Generate #include line */
-#define	GEN_INCLUDE_STD(typename)	GEN_INCLUDE("<" typename ".h>")
+#define	GEN_INCLUDE_STD(typename)	do {			\
+	if((arg->flags & A1C_INCLUDES_QUOTED)) {			\
+		GEN_INCLUDE("\"" typename ".h\"");		\
+	} else {						\
+		GEN_INCLUDE("<" typename ".h>");		\
+	} } while(0)
 #define	GEN_INCLUDE(filename)	do {				\
 	int saved_target = arg->target->target;			\
 	if(!filename) break;					\
@@ -107,4 +113,21 @@ int asn1c_compiled_output(arg_t *arg, const char *fmt, ...);
 	REDIR(saved_target);					\
 } while(0)
 
+/*
+ * Format LONG_MIN according to C90 rules.
+ */
+#define OINT(iv)	do {					\
+	if(iv == (-2147483647L - 1))				\
+		OUT("(-2147483647L - 1)");			\
+	else							\
+		OUT("%" PRIdASN, iv);				\
+} while(0)
+
+#define OINTS(iv)	do {					\
+	if(iv == (-2147483647L - 1))				\
+		OUT("(-2147483647L - 1)");			\
+	else							\
+		OUT("% " PRIdASN, iv);				\
+} while(0)
+
 #endif	/* _ASN1_COMPILED_OUTPUT_H_ */
