@@ -2,6 +2,6 @@
 #define	_ASN1C_SAVE_H_
 
 int asn1c_save_compiled_output(arg_t *arg, const char *datadir,
-	int argc, char **argv);
+	int argc, int optc, char **argv);
 
 #endif	/* _ASN1C_SAVE_H_ */
