@@ -81,6 +81,8 @@ asn1f_printable_value(asn1p_value_t *v) {
 		/* Buffer is guaranteed to be null-terminated */
 		assert(v->value.string.buf[v->value.string.size] == '\0');
 		return (char *)v->value.string.buf;
+	case ATV_TYPE:
+		return "<Type>";
 	case ATV_BITVECTOR:
 		{
 			uint8_t *bitvector;
@@ -160,6 +162,8 @@ asn1f_printable_value(asn1p_value_t *v) {
 			assert(reflen == (size_t)(ptr - managedptr));
 			return managedptr;
 		}
+	case ATV_VALUESET:
+		return "<ValueSet>";
 	case ATV_CHOICE_IDENTIFIER:
 		{
 			char *cid = v->value.choice_identifier.identifier;
@@ -196,6 +200,17 @@ asn1f_recurse_expr(arg_t *arg, int (*callback)(arg_t *arg)) {
 
 	assert(expr);
 
+	if(expr->lhs_params && expr->spec_index == -1) {
+		int i;
+		for(i = 0; i < expr->specializations.pspecs_count; i++) {
+			arg->expr = expr->specializations.pspec[i].my_clone;
+			ret = asn1f_recurse_expr(arg, callback);
+			RET2RVAL(ret, rvalue);
+		}
+		arg->expr = expr;       /* revert */
+		return rvalue;
+	}
+
 	/*
 	 * Invoke the callback at this very level.
 	 */
@@ -223,15 +238,14 @@ asn1f_recurse_expr(arg_t *arg, int (*callback)(arg_t *arg)) {
  * Check that every child of a given expr has unique name or does not have any.
  */
 int
-asn1f_check_unique_expr(arg_t *arg,
-		int (*opt_compare)(asn1p_expr_t *a, asn1p_expr_t *b)) {
+asn1f_check_unique_expr(arg_t *arg) {
 	asn1p_expr_t *expr;
 	int rvalue = 0;
 
 	TQ_FOR(expr, &(arg->expr->members), next) {
 		if(expr->Identifier) {
 			int ret = asn1f_check_unique_expr_child(arg, expr,
-				opt_compare);
+				0, "identifier");
 			if(ret) rvalue = -1;
 		} else {
 			/*
@@ -250,9 +264,11 @@ asn1f_check_unique_expr(arg_t *arg,
  */
 int
 asn1f_check_unique_expr_child(arg_t *arg, asn1p_expr_t *child,
-		int (*opt_compare)(asn1p_expr_t *a, asn1p_expr_t *b)) {
+		int (*opt_compare)(asn1p_expr_t *a, asn1p_expr_t *b),
+		const char *opt_property_name) {
 	asn1p_expr_t *expr;
-	int rvalue = 0;
+
+	if(!opt_property_name) opt_property_name = "property";
 
 	assert(child);
 	assert(opt_compare || child->Identifier);
@@ -277,27 +293,20 @@ asn1f_check_unique_expr_child(arg_t *arg, asn1p_expr_t *child,
 		}
 
 		if(ret == 0) {
-			char *msg;
-			msg = opt_compare
-				?"Expressions clash"
-				:"Identifiers name clash";
-			arg->eh(1,
-				"%s: "
+			FATAL("Clash detected: "
 				"\"%s\" at line %d has similar %s with "
 				"\"%s\" at line %d",
-				msg,
 				expr->Identifier,
 				expr->_lineno,
-				opt_compare?"property":"name",
+				opt_property_name,
 				child->Identifier,
 				child->_lineno
 			);
-
-			rvalue = -1;
+			return -1;
 		}
 	}
 
-	return rvalue;
+	return 0;
 }
 
 int
