@@ -29,6 +29,30 @@ asn1p_value_fromref(asn1p_ref_t *ref, int do_copy) {
 	}
 }
 
+asn1p_value_t *
+asn1p_value_fromconstr(asn1p_constraint_t *ct, int do_copy) {
+	if(ct) {
+		asn1p_value_t *v = calloc(1, sizeof *v);
+		if(v) {
+			if(do_copy) {
+				v->value.constraint
+					= asn1p_constraint_clone(ct);
+				if(v->value.constraint == NULL) {
+					free(v);
+					return NULL;
+				}
+			} else {
+				v->value.constraint = ct;
+			}
+			v->type = ATV_VALUESET;
+		}
+		return v;
+	} else {
+		errno = EINVAL;
+		return NULL;
+	}
+}
+
 asn1p_value_t *
 asn1p_value_frombits(uint8_t *bits, int size_in_bits, int do_copy) {
 	if(bits) {
@@ -108,8 +132,25 @@ asn1p_value_fromint(asn1c_integer_t i) {
 	return v;
 }
 
+asn1p_value_t *
+asn1p_value_fromtype(asn1p_expr_t *expr) {
+	asn1p_value_t *v = calloc(1, sizeof *v);
+	if(v) {
+		v->value.v_type = expr;
+		v->type = ATV_TYPE;
+	}
+	return v;
+}
+
 asn1p_value_t *
 asn1p_value_clone(asn1p_value_t *v) {
+	return asn1p_value_clone_with_resolver(v, 0, 0);
+}
+
+asn1p_value_t *
+asn1p_value_clone_with_resolver(asn1p_value_t *v,
+		asn1p_value_t *(*resolver)(asn1p_value_t *, void *rarg),
+		void *rarg) {
 	asn1p_value_t *clone = NULL;
 	if(v) {
 		switch(v->type) {
@@ -118,6 +159,8 @@ asn1p_value_clone(asn1p_value_t *v) {
 			return calloc(1, sizeof(*clone));
 		case ATV_REAL:
 			return asn1p_value_fromdouble(v->value.v_double);
+		case ATV_TYPE:
+			return asn1p_value_fromtype(v->value.v_type);
 		case ATV_INTEGER:
 		case ATV_MIN:
 		case ATV_MAX:
@@ -142,7 +185,19 @@ asn1p_value_clone(asn1p_value_t *v) {
 			return asn1p_value_frombits(v->value.binary_vector.bits,
 				v->value.binary_vector.size_in_bits, 1);
 		case ATV_REFERENCED:
+			if(resolver) {
+				clone = resolver(v, rarg);
+				if(clone) return clone;
+				else if(errno != ESRCH) return NULL;
+			}
 			return asn1p_value_fromref(v->value.reference, 1);
+		case ATV_VALUESET:
+			if(resolver) {
+				clone = resolver(v, rarg);
+				if(clone) return clone;
+				else if(errno != ESRCH) return NULL;
+			}
+			return asn1p_value_fromconstr(v->value.constraint, 1);
 		case ATV_CHOICE_IDENTIFIER: {
 			char *id = v->value.choice_identifier.identifier;
 			clone = calloc(1, sizeof(*clone));
@@ -170,6 +225,9 @@ asn1p_value_free(asn1p_value_t *v) {
 		case ATV_NOVALUE:
 		case ATV_NULL:
 			break;
+		case ATV_TYPE:
+			asn1p_expr_free(v->value.v_type);
+			break;
 		case ATV_REAL:
 		case ATV_INTEGER:
 		case ATV_MIN:
@@ -192,6 +250,9 @@ asn1p_value_free(asn1p_value_t *v) {
 		case ATV_REFERENCED:
 			asn1p_ref_free(v->value.reference);
 			break;
+		case ATV_VALUESET:
+			asn1p_constraint_free(v->value.constraint);
+			break;
 		case ATV_CHOICE_IDENTIFIER:
 			free(v->value.choice_identifier.identifier);
 			asn1p_value_free(v->value.choice_identifier.value);
